from django.utils.translation import gettext_lazy as _
from rest_framework import serializers


class RefreshTokenSerializer(serializers.Serializer):
    refresh_token = serializers.CharField(label=_("Refresh token"))



from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import AuthenticationForm
from django.utils.translation import gettext_lazy as _

from datetime import timedelta

from garpix_utils.logs.enums.get_enums import Action, ActionResult
from garpix_utils.logs.loggers import ib_logger
from garpix_utils.logs.services.logger_iso import LoggerIso

from garpix_user.utils.current_date import set_current_date
from garpix_user.utils.get_password_settings import get_password_settings

User = get_user_model()


class LoginForm(AuthenticationForm):
    username = forms.CharField()
    password = forms.CharField(widget=forms.PasswordInput)

    def clean(self):

        password_validity_period = get_password_settings()['password_validity_period']

        super(LoginForm, self).clean()
        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')
        valid = False
        if username and password:
            user = User.objects.filter(username=username.lower()).first()

            if not user or user.keycloak_auth_only:
                raise forms.ValidationError(_('User is not found'))

            if user and not user.is_active:
                raise forms.ValidationError(_('User is inactive. You must confirm the registration email address at registration.'))

            valid = user.check_password(password)

            if not valid:
                raise forms.ValidationError(_('Invalid: username / password'))

            if user and user.is_blocked:
                raise forms.ValidationError(_("Your account is blocked. Please contact your administrator"))
            if user and password_validity_period != -1 and not user.keycloak_auth_only and user.password_updated_date + timedelta(
                    days=password_validity_period) <= set_current_date():
                raise forms.ValidationError(_('Your password has expired. Please change password'))

            message = f'Пользователь {user.username} вошел в систему.'

            log = ib_logger.create_log(action=Action.user_login.value,
                                       obj=get_user_model().__name__,
                                       obj_address=self.request.path,
                                       result=ActionResult.success,
                                       sbj=user.username,
                                       sbj_address=LoggerIso.get_client_ip(self.request),
                                       msg=message)

            ib_logger.write_string(log)

        return valid



import jwt
from django.db import IntegrityError
from rest_framework.authentication import TokenAuthentication
from django.contrib.auth import get_user_model
from django.utils import timezone
from django.conf import settings
from datetime import timedelta

from garpix_user.utils.get_password_settings import get_password_settings
from garpix_user.utils.get_token_from_request import get_token_from_request
from django.contrib.auth.models import AnonymousUser
from django.utils.translation import gettext as _


def get_user_by_token(token):
    from ..models.access_token import AccessToken as Token
    from oauth2_provider.models import AccessToken
    User = get_user_model()

    access_token_ttl_seconds = get_password_settings()['access_token_ttl_seconds']

    # Refresh django rest token
    try:
        tok = Token.objects.get(key=token)
        if access_token_ttl_seconds > 0:
            if tok.created + timedelta(seconds=access_token_ttl_seconds) < timezone.now():
                tok.delete()
                raise Exception("Token expired.")
        user = User.active_objects.get(id=tok.user_id)
        return user
    except:  # noqa
        pass

    # Refresh social auth token
    try:
        tok = AccessToken.objects.get(token=token)
        if access_token_ttl_seconds > 0:
            if tok.created + timedelta(seconds=access_token_ttl_seconds) < timezone.now():
                tok.delete()
        else:
            user = tok.user
            return user
    except:  # noqa
        pass

    return AnonymousUser()


def get_user_by_jwt_token(token):

    access_token_ttl_seconds = get_password_settings()['access_token_ttl_seconds']

    User = get_user_model()

    jwt_secret_key = settings.GARPIX_USER.get('JWT_SECRET_KEY', None)

    if jwt_secret_key is None:
        raise IntegrityError(_('JWT_SECRET_KEY is not set'))

    try:
        token_data = jwt.decode(token, jwt_secret_key, algorithms='HS256', verify=True)

        if access_token_ttl_seconds > 0:
            if token_data['token_created_at'] + timedelta(
                    seconds=access_token_ttl_seconds) < timezone.now():
                raise Exception("Token expired.")
        return User.active_objects.get(username=token_data['username'], is_blocked=False)
    except Exception as e:
        print(str(e))

    return AnonymousUser()


class MainAuthentication(TokenAuthentication):

    keyword = 'Bearer'

    def authenticate(self, request):

        token = get_token_from_request(request, keyword=self.keyword)
        if token is None:
            return None

        use_jwt = settings.GARPIX_USER.get('REST_AUTH_TOKEN_JWT', False)

        if use_jwt:
            user = get_user_by_jwt_token(token)
        else:
            user = get_user_by_token(token)

        if user is not None:
            return user, None
        return user, None



from django.contrib import admin
from django.contrib.auth.admin import GroupAdmin
from django.contrib.auth.models import Group
from garpix_utils.logs.enums.get_enums import Action
from garpix_utils.logs.loggers import ib_logger
from garpix_utils.logs.mixins.create_log import CreateLogMixin

admin.site.unregister(Group)


@admin.register(Group)
class GarpixGroupAdmin(CreateLogMixin, GroupAdmin):

    def save_model(self, request, obj, form, change):
        log = self.log_change_or_create(ib_logger, request, obj, change,
                                        action_change=Action.group_change.value,
                                        action_create=Action.group_create.value)
        super().save_model(request, obj, form, change)
        ib_logger.write_string(log)

    def save_related(self, request, form, formsets, change):
        if change:
            log = self.log_change_m2m_field(ib_logger, request, super(), form, formsets, change,
                                            action_change=Action.group_change.value,)
            ib_logger.write_string(log)
        else:
            super().save_related(request, form, formsets, change)

    def delete_model(self, request, obj):
        action = Action.group_delete.value
        log = self.log_delete(ib_logger, request, obj, action)
        super().delete_model(request, obj)
        ib_logger.write_string(log)

    def delete_queryset(self, request, queryset):
        action = Action.group_delete.value
        logs = []
        for obj in queryset:
            logs.append(self.log_delete(ib_logger, request, obj, action))
        super().delete_queryset(request, queryset)
        for log in logs:
            ib_logger.write_string(log)



from drf_spectacular.utils import extend_schema
from rest_framework import viewsets, status
from rest_framework.decorators import action
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from django.utils.translation import gettext_lazy as _

from garpix_user.models import UserSession
from garpix_user.serializers import RestorePasswordSerializer, RestoreCheckCodeSerializer, RestoreSetPasswordSerializer
from garpix_user.utils.drf_spectacular import user_session_token_header_parameter


@extend_schema(
    parameters=[
        user_session_token_header_parameter()
    ]
)
class RestorePasswordView(viewsets.ViewSet):

    def get_serializer_class(self):
        if self.action == 'send_code':
            return RestorePasswordSerializer
        if self.action == 'check_code':
            return RestoreCheckCodeSerializer
        return RestoreSetPasswordSerializer

    @extend_schema(summary=_('Restore password. Step 1'))
    @action(methods=['POST'], detail=False)
    def send_code(self, request, *args, **kwargs):
        serializer = self.get_serializer_class()(data=request.data)
        serializer.is_valid(raise_exception=True)

        user = UserSession.get_from_request(request)

        if not user:
            return Response({"non_field_errors": [_("user-session-token not set")]}, status=status.HTTP_400_BAD_REQUEST)

        result, error = user.send_restore_code(username=serializer.validated_data['username'])

        if not result:
            error.raise_exception(exception_class=ValidationError)
        return Response({"result": "success"})

    @extend_schema(summary=_('Restore password. Step 2'))
    @action(methods=['POST'], detail=False)
    def check_code(self, request, *args, **kwargs):

        user = UserSession.get_from_request(request)

        if not user:
            return Response({"non_field_errors": [_("user-session-token not set")]}, status=status.HTTP_400_BAD_REQUEST)

        serializer = self.get_serializer_class()(data=request.data)
        serializer.is_valid(raise_exception=True)

        result, error = user.check_restore_code(username=serializer.validated_data['username'],
                                                restore_password_confirm_code=serializer.data[
                                                    'restore_password_confirm_code'])
        if not result:
            error.raise_exception(exception_class=ValidationError)
        return Response({"result": "success"})

    @extend_schema(summary=_('Restore password. Step 3'))
    @action(methods=['POST'], detail=False)
    def set_password(self, request, *args, **kwargs):

        user = UserSession.get_from_request(request)

        if not user:
            return Response({"non_field_errors": [_("user-session-token not set")]}, status=status.HTTP_400_BAD_REQUEST)

        serializer = self.get_serializer_class()(data=request.data)
        serializer.is_valid(raise_exception=True)

        result, error = user.restore_password(new_password=serializer.data['new_password'],
                                              username=serializer.validated_data['username'],
                                              restore_password_confirm_code=serializer.data[
                                                  'restore_password_confirm_code'])

        if not result:
            error.raise_exception(exception_class=ValidationError)
        return Response({"result": "success"})



from django.conf import settings
from django.urls import reverse
from django.test import override_settings

from garpix_user.models import UserSession
from garpix_user.tests.utils.settings import GARPIX_USER_SETTINGS, GARPIX_USER_CONFIRMATION_SETTINGS
from garpix_user.tests.utils.test_case_mixin import ApiTestMixin
from django.utils.translation import gettext_lazy_lazy as _

BASE_GARPIX_USER_SETTINGS = getattr(settings, 'GARPIX_USER', dict())


@override_settings(GARPIX_USER=GARPIX_USER_SETTINGS)
class RegistrationApiTest(ApiTestMixin):
    def setUp(self):
        self.set_init_data()

    def test_registration_by_username(self):
        """
        Check registration by username
        """

        self.User.USERNAME_FIELDS = ('username',)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'username': self.new_user_data['username'],
                'password': self.new_user_data['password'],
                'password_2': self.new_user_data['password']
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 201)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.new_user_data['username'],
                'password': self.new_user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.check_login_result(response)

    def test_registration_by_email(self):
        """
        Check registration by email
        """

        self.User.USERNAME_FIELDS = ('email',)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'email': self.new_user_data['email'],
                'password': self.new_user_data['password'],
                'password_2': self.new_user_data['password']
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 201)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.new_user_data['email'],
                'password': self.new_user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.check_login_result(response)

    def test_registration_by_phone(self):
        """
        Check registration by phone number
        """

        self.User.USERNAME_FIELDS = ('phone',)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'phone': self.new_user_data['phone'],
                'password': self.new_user_data['password'],
                'password_2': self.new_user_data['password']
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 201)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.new_user_data['phone'],
                'password': self.new_user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.check_login_result(response)

    def test_registration_by_phone_or_email(self):
        """
        Check registration by phone number or email if User.USERNAME_FIELDS = ('phone', 'email')
        """

        self.User.USERNAME_FIELDS = ('phone', 'email')

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'phone': self.new_user_data['phone'],
                'email': self.new_user_data['email'],
                'password': self.new_user_data['password'],
                'password_2': self.new_user_data['password']
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 201)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.new_user_data['phone'],
                'password': self.new_user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.check_login_result(response)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.new_user_data['email'],
                'password': self.new_user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.check_login_result(response)

    def test_registration_incorrect_password(self):
        """
        Check registration failed if the password is incorrect
        """

        self.User.USERNAME_FIELDS = ('phone',)

        # password don't match
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'phone': self.new_user_data['phone'],
                'password': self.new_user_data['password'],
                'password_2': '1q2W3e4R5t6Y'
            },
            HTTP_ACCEPT='application/json'
        )

        self.assertEqual(response.status_code, 400)
        self.assertDictEqual(response.json(), {'password_2': [_('Passwords do not match')]})

        # incorrect password
        min_length = GARPIX_USER_SETTINGS.get('MIN_LENGTH_PASSWORD', 8)
        min_digits = GARPIX_USER_SETTINGS.get('MIN_DIGITS_PASSWORD', 2)
        min_chars = GARPIX_USER_SETTINGS.get('MIN_CHARS_PASSWORD', 2)
        min_uppercase = GARPIX_USER_SETTINGS.get('MIN_UPPERCASE_PASSWORD', 1)

        # check for min length
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'phone': self.new_user_data['phone'],
                'password': 'qwerty',
                'password_2': 'qwerty'
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 400)
        self.assertDictEqual(response.json(), {
            'password': [_('Password must be at least {min_length} characters long.'.format(min_length=min_length))]})

        # check for min digits
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'phone': self.new_user_data['phone'],
                'password': 'qwertyq1',
                'password_2': 'qwertyq1'
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 400)
        self.assertDictEqual(response.json(), {
            'password': [_('Password must container at least {min_digits} digits.'.format(min_digits=min_digits))]})

        # check for min char
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'phone': self.new_user_data['phone'],
                'password': '12345678',
                'password_2': '12345678'
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 400)
        self.assertDictEqual(response.json(), {
            'password': [_('Password must container at least {min_chars} chars.'.format(min_chars=min_chars))]})

        # check for uppercase letter
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'phone': self.new_user_data['phone'],
                'password': 'q1w2e3r4t5y6',
                'password_2': 'q1w2e3r4t5y6'
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 400)
        self.assertDictEqual(response.json(), {'password': [
            _('Password must container at least {min_uppercase} uppercase letter.'.format(
                min_uppercase=min_uppercase))]})

    @override_settings(GARPIX_USER=GARPIX_USER_CONFIRMATION_SETTINGS)
    def test_registration_unconfirmed_data(self):
        """
        Check registration failed if email and/or phone number was not confirmed before registrations, but
        the USE_PREREGISTRATION_EMAIL_CONFIRMATION and/or USE_PREREGISTRATION_PHONE_CONFIRMATION settings were set
        """

        # email
        self.User.USERNAME_FIELDS = ('email',)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'email': self.new_user_data['email'],
                'password': self.new_user_data['password'],
                'password_2': self.new_user_data['password']
            },
            HTTP_ACCEPT='application/json'
        )

        self.assertEqual(response.status_code, 400)
        self.assertDictEqual(response.json(), {'email': [_('Email was not confirmed')]})

        # phone
        self.User.USERNAME_FIELDS = ('phone',)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'phone': self.new_user_data['phone'],
                'password': self.new_user_data['password'],
                'password_2': self.new_user_data['password']
            },
            HTTP_ACCEPT='application/json'
        )

        self.assertEqual(response.status_code, 400)
        self.assertDictEqual(response.json(), {'phone': [_('Phone number was not confirmed')]})

    def test_registration_used_data(self):
        """
        Check registration failed if phone and/or email was already registered
        """

        self.User.USERNAME_FIELDS = ('phone', 'email')

        # email

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'email': self.user_data['email'],
                'password': self.new_user_data['password'],
                'password_2': self.new_user_data['password']
            },
            HTTP_ACCEPT='application/json'
        )

        self.assertEqual(response.status_code, 400)
        self.assertDictEqual(response.json(), {'email': [_('This email is already in use')]})

        # phone

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_registration'),
            {
                'phone': self.user_data['phone'],
                'password': self.new_user_data['password'],
                'password_2': self.new_user_data['password']
            },
            HTTP_ACCEPT='application/json'
        )

        self.assertEqual(response.status_code, 400)
        self.assertDictEqual(response.json(), {'phone': [_('This phone is already in use')]})

    @override_settings(GARPIX_USER=BASE_GARPIX_USER_SETTINGS)
    def test_registration_after_email_confirmation(self):
        """
        Check registration is OK after email confirmation
        """

        if BASE_GARPIX_USER_SETTINGS.get('USE_EMAIL_CONFIRMATION', False) and BASE_GARPIX_USER_SETTINGS.get('USE_PREREGISTRATION_EMAIL_CONFIRMATION', False):
            self.User.USERNAME_FIELDS = ('email',)

            response = self.client.post(
                reverse('garpix_user:garpix_user_api:api_confirm_email-send-code'),
                {
                    'email': self.new_user_data['email']
                },
                HTTP_ACCEPT='application/json'
            )

            self.assertEqual(response.status_code, 200)
            self.assertIn('token_number', response.json())

            token = response.json()['token_number']

            user_session = UserSession.objects.get(token_number=token)

            if BASE_GARPIX_USER_SETTINGS.get('USE_EMAIL_LINK_CONFIRMATION', False):
                user_session.confirm_email(email_confirmation_code=user_session.email_confirmation_code)
                user_session.save()
            else:

                response = self.client.post(
                    reverse('garpix_user:garpix_user_api:api_confirm_email-check-code'),
                    {
                        'email_confirmation_code': user_session.email_confirmation_code
                    },
                    HTTP_ACCEPT='application/json',
                    **{f'HTTP_{UserSession.HEAD_NAME}': token}
                )

                self.assertEqual(response.status_code, 204)

            response = self.client.post(
                reverse('garpix_user:garpix_user_api:api_registration'),
                {
                    'email': self.new_user_data['email'],
                    'password': self.new_user_data['password'],
                    'password_2': self.new_user_data['password']
                },
                HTTP_ACCEPT='application/json',
                **{f'HTTP_{UserSession.HEAD_NAME}': token}
            )
            self.assertEqual(response.status_code, 201)

    @override_settings(GARPIX_USER=BASE_GARPIX_USER_SETTINGS)
    def test_registration_after_phone_confirmation(self):
        """
        Check registration is OK after phone confirmation
        """

        if BASE_GARPIX_USER_SETTINGS.get('USE_PHONE_CONFIRMATION', False) and BASE_GARPIX_USER_SETTINGS.get('USE_PREREGISTRATION_PHONE_CONFIRMATION', False):
            self.User.USERNAME_FIELDS = ('phone',)

            response = self.client.post(
                reverse('garpix_user:garpix_user_api:api_confirm_phone-send-code'),
                {
                    'phone': self.new_user_data['phone']
                },
                HTTP_ACCEPT='application/json'
            )

            self.assertEqual(response.status_code, 200)
            self.assertIn('token_number', response.json())

            token = response.json()['token_number']

            user_session = UserSession.objects.get(token_number=token)

            response = self.client.post(
                reverse('garpix_user:garpix_user_api:api_confirm_phone-check-code'),
                {
                    'phone_confirmation_code': user_session.phone_confirmation_code
                },
                HTTP_ACCEPT='application/json',
                **{f'HTTP_{UserSession.HEAD_NAME}': token}
            )

            self.assertEqual(response.status_code, 200)

            response = self.client.post(
                reverse('garpix_user:garpix_user_api:api_registration'),
                {
                    'phone': self.new_user_data['phone'],
                    'password': self.new_user_data['password'],
                    'password_2': self.new_user_data['password']
                },
                HTTP_ACCEPT='application/json',
                **{f'HTTP_{UserSession.HEAD_NAME}': token}
            )
            self.assertEqual(response.status_code, 201)



from django.conf import settings
from django.contrib.auth.hashers import check_password
from drf_spectacular.utils import extend_schema
from rest_framework import viewsets, status
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated, AllowAny
from rest_framework.response import Response
from django.utils.translation import gettext_lazy as _

from garpix_user.mixins.views.auth_token_mixin import AuthTokenViewMixin
from garpix_user.models import PasswordHistory
from garpix_user.permissions import IsUnAuthenticated
from garpix_user.serializers import ChangePasswordSerializer
from garpix_user.serializers.passwrod_serializer import ChangePasswordUnauthorizedSerializer
from garpix_user.utils.drf_spectacular import user_session_token_header_parameter
from garpix_user.utils.get_password_settings import get_password_settings


@extend_schema(
    parameters=[
        user_session_token_header_parameter()
    ]
)
class ChangePasswordView(AuthTokenViewMixin, viewsets.ViewSet):

    def get_serializer_class(self):
        if self.action == 'change_password_unauthorized':
            return ChangePasswordUnauthorizedSerializer
        return ChangePasswordSerializer

    permission_classes = [IsAuthenticated]

    @extend_schema(summary=_('Change password (for authorized users)'))
    @action(methods=['POST'], detail=False)
    def change_password(self, request, *args, **kwargs):

        user = request.user

        if user.keycloak_auth_only:
            return Response({"new_password": [_('You can not change your password. Please contact administrator')]},
                            status=status.HTTP_400_BAD_REQUEST)

        serializer = self.get_serializer_class()(data=request.data, context={"request": request})
        serializer.is_valid(raise_exception=True)

        password_history = get_password_settings()['password_history']
        if password_history != -1:
            password_history_rows = PasswordHistory.objects.filter(user=user).order_by(
                '-created_at')[:password_history + 1].values_list('password', flat=True)
            for _pass in password_history_rows:
                if check_password(request.data['new_password'], _pass):
                    return Response({"new_password": [_('You can not use the same password you already had to')]},
                                    status=status.HTTP_400_BAD_REQUEST)
        user.set_password(request.data['new_password'])
        user.save()

        return Response({"result": "success"})

    @extend_schema(summary=_('Change password (for unauthorized users)'))
    @action(methods=['POST'], detail=False, permission_classes=[IsUnAuthenticated])
    def change_password_unauthorized(self, request, *args, **kwargs):

        serializer = self.get_serializer_class()(data=request.data, context={"request": request})
        serializer.is_valid(raise_exception=True)

        user = serializer.validated_data['user']

        if user.keycloak_auth_only:
            return Response({"new_password": [_('You can not change your password. Please contact administrator')]},
                            status=status.HTTP_400_BAD_REQUEST)

        password_history = get_password_settings()['password_history']
        if password_history != -1:
            password_history_rows = PasswordHistory.objects.filter(user=user).order_by(
                '-created_at')[:password_history + 1].values_list('password', flat=True)
            for _pass in password_history_rows:
                if check_password(request.data['new_password'], _pass):
                    return Response({"new_password": [_('You can not use the same password you already had to')]},
                                    status=status.HTTP_400_BAD_REQUEST)
        user.set_password(request.data['new_password'])
        user.needs_password_update = False
        user.save()

        use_jwt = settings.GARPIX_USER.get('REST_AUTH_TOKEN_JWT', False)

        if use_jwt:
            return self._get_jwt_data(user)
        return self._get_access_token_data(user)



from django.conf import settings
from django.core.validators import ValidationError
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers

from garpix_user.utils.repluralize import rupluralize


class ModelException(Exception):
    code = 'invalid'
    field = None

    def __init__(self, field=None, extra_data={}):
        self.message = self.get_message().format(**extra_data)
        self.field = field

    def raise_exception(self, exception_class=ValidationError):
        field = self.field or 'non_field_errors'
        raise exception_class({field: [self.message]}, code=self.code)

    def get_message(self):
        return ''


class WaitException(ModelException):
    def get_message(self):
        return settings.GARPIX_USER.get('WAIT_RESPONSE',
                                        _("Less than {min_time} {minutes} has passed since the last request")).format(
            min_time=settings.GARPIX_USER.get('TIME_LAST_REQUEST', 1),
            minutes=rupluralize(settings.GARPIX_USER.get('TIME_LAST_REQUEST', 1), _('minute,minutes')))


class UserRegisteredException(ModelException):
    def get_message(self):
        return settings.GARPIX_USER.get('USER_REGISTERED_RESPONSE',
                                        _("User with such data has been already registered"))


class UserUnregisteredException(ModelException):
    def get_message(self):
        return settings.GARPIX_USER.get('USER_UNREGISTERED_RESPONSE', _("User with such data has not been registered"))


class IncorrectCodeException(ModelException):
    def get_message(self):
        return _("Incorrect code")


class NoTimeLeftException(ModelException):
    def get_message(self):
        return _("Code has expired. Request it again")


class NotAuthenticateException(ModelException):
    def get_message(self):
        return _("Credentials were not provided")


class NotConfirmedException(ModelException):
    def get_message(self):
        return _("{field} was not confirmed")


class ValidationErrorSerializer(serializers.Serializer):
    field = serializers.ListField(default=['Error'])



from django.conf import settings
from django.test import TestCase
from django.contrib.auth import get_user_model
from django.urls import reverse

from garpix_user.models import ReferralType

User = get_user_model()
BASE_GARPIX_USER_SETTINGS = getattr(settings, 'GARPIX_USER', dict())


class LoginViewTest(TestCase):
    def setUp(self):
        self.username = 'testuser1'
        self.password = '12345'
        self.user = User.objects.create_user(username=self.username, password=self.password)
        self.user.save()

    def test_invalid_username_password(self):
        response = self.client.post(
            reverse('authorize'),
            {
                'username': self.username,
                'password': 'passwordtest',
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.content, b'{"__all__": ["Invalid: username / password"]}')

    def test_valid_username_password(self):
        response = self.client.post(
            reverse('authorize'),
            {
                'username': self.username,
                'password': self.password,
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 200)

    def test_valid_redirects(self):
        response = self.client.post(
            reverse('authorize'),
            {
                'username': self.username,
                'password': self.password,
            },
        )
        self.assertRedirects(response, '/')

    def test_referral_links(self):
        """
        Check referral links functionality
        """
        if BASE_GARPIX_USER_SETTINGS.get('USE_REFERRAL_LINKS', False):
            referral_link = ReferralType.objects.create(title='test_link')
            response = self.client.get(
                reverse('garpix_user:referral_link', args=[referral_link.referral_hash])
            )
            self.assertEqual(response.status_code, 302)
            self.assertRedirects(response,
                                 f"{BASE_GARPIX_USER_SETTINGS.get('REFERRAL_REDIRECT_URL', '/')}?status=success")



import binascii
import os
from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _


class RefreshToken(models.Model):
    key = models.CharField(_("Key"), max_length=40, primary_key=True)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='refresh_tokens',
        on_delete=models.CASCADE, verbose_name=_("User")
    )
    created = models.DateTimeField(_("Created"), auto_now_add=True)

    class Meta:
        verbose_name = _("Токен обновления доступа | Refresh Token")
        verbose_name_plural = _("Токены обновления доступа | Refresh Tokens")

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = self.generate_key()
        return super().save(*args, **kwargs)

    @classmethod
    def generate_key(cls):
        return binascii.hexlify(os.urandom(20)).decode()

    def __str__(self):
        return self.key



from django.urls import reverse
from django.test import override_settings
import time

from garpix_user.tests.utils.test_case_mixin import ApiTestMixin
from django.utils.translation import gettext_lazy_lazy as _


class LoginApiTest(ApiTestMixin):
    def setUp(self):
        self.set_init_data()

    def test_invalid_username_password(self):
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.user_data['username'],
                'password': 'passwordtest',
            },
            HTTP_ACCEPT='application/json'
        )

        self.assertDictEqual(response.json(), {"non_field_errors": [_("Unable to log in with provided credentials.")]})

    def test_valid_username_password(self):
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.user_data['username'],
                'password': self.user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.check_login_result(response)

    def test_access_to_protected_view(self):
        response = self.client.get(reverse('current-user'), HTTP_ACCEPT='application/json')
        self.assertDictEqual(response.json(), {'detail': _('Authentication credentials were not provided.')})
        # auth
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.user_data['username'],
                'password': self.user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 200)
        self.assertIn('access_token', response.json())
        access_token = response.json()['access_token']
        # get protected view
        response = self.client.get(reverse('current-user'), HTTP_ACCEPT='application/json',
                                   HTTP_AUTHORIZATION=f'Bearer {access_token}')
        self.assertDictEqual(response.json(), {'username': self.user_data['username']})

    def test_access_to_protected_view_after_logout(self):
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.user_data['username'],
                'password': self.user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 200)
        self.assertIn('access_token', response.json())
        access_token = response.json()['access_token']
        # get protected view
        response = self.client.get(reverse('current-user'), HTTP_ACCEPT='application/json',
                                   HTTP_AUTHORIZATION=f'Bearer {access_token}')
        self.assertDictEqual(response.json(), {'username': self.user_data['username']})
        # logout
        response = self.client.post(reverse('garpix_user:garpix_user_api:api_logout'), HTTP_ACCEPT='application/json',
                                    HTTP_AUTHORIZATION=f'Bearer {access_token}')
        self.assertDictEqual(response.json(), {'result': True})
        # get protected view after logout
        response = self.client.get(reverse('current-user'), HTTP_ACCEPT='application/json')
        self.assertDictEqual(response.json(), {'detail': _('Authentication credentials were not provided.')})

    @override_settings(GARPIX_ACCESS_TOKEN_TTL_SECONDS=5)
    def test_refresh_token(self):
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.user_data['username'],
                'password': self.user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 200)
        self.assertIn('access_token', response.json())
        access_token1 = response.json()['access_token']
        refresh_token = response.json()['refresh_token']
        # get protected view
        response = self.client.get(reverse('current-user'), HTTP_ACCEPT='application/json',
                                   HTTP_AUTHORIZATION=f'Bearer {access_token1}')
        self.assertDictEqual(response.json(), {'username': self.user_data['username']})
        # wait and get protected view with expired access token
        time.sleep(5)
        response = self.client.get(reverse('current-user'), HTTP_ACCEPT='application/json')
        self.assertDictEqual(response.json(), {'detail': _('Authentication credentials were not provided.')})
        # refresh token
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_refresh'),
            {
                'refresh_token': refresh_token,
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 200)
        self.assertIn('access_token', response.json())
        access_token2 = response.json()['access_token']
        self.assertNotEqual(access_token1, access_token2)
        # get protected view
        response = self.client.get(reverse('current-user'), HTTP_ACCEPT='application/json',
                                   HTTP_AUTHORIZATION=f'Bearer {access_token2}')
        self.assertDictEqual(response.json(), {'username': self.user_data['username']})

    @override_settings(GARPIX_REFRESH_TOKEN_TTL_SECONDS=5)
    def test_refresh_token_expired(self):
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.user_data['username'],
                'password': self.user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 200)
        self.assertIn('access_token', response.json())
        access_token = response.json()['access_token']
        refresh_token = response.json()['refresh_token']
        # get protected view
        response = self.client.get(reverse('current-user'), HTTP_ACCEPT='application/json',
                                   HTTP_AUTHORIZATION=f'Bearer {access_token}')
        self.assertDictEqual(response.json(), {'username': self.user_data['username']})
        # refresh token
        time.sleep(5)
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_refresh'),
            {
                'refresh_token': refresh_token,
            },
            HTTP_ACCEPT='application/json'
        )
        self.assertEqual(response.status_code, 200)
        self.assertDictEqual(response.json(), {'result': False})

    def test_login_by_email(self):
        self.User.USERNAME_FIELDS = ('email',)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.user_data['email'],
                'password': self.user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.check_login_result(response)

    def test_login_by_phone(self):
        self.User.USERNAME_FIELDS = ('phone',)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.user_data['phone'],
                'password': self.user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.check_login_result(response)

    def test_login_by_phone_or_email(self):
        self.User.USERNAME_FIELDS = ('phone', 'email')

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.user_data['phone'],
                'password': self.user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.check_login_result(response)

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': self.user_data['email'],
                'password': self.user_data['password'],
            },
            HTTP_ACCEPT='application/json'
        )
        self.check_login_result(response)



from rest_framework.permissions import BasePermission


class IsUnAuthenticated(BasePermission):
    """
    Allows access only to not authenticated users.
    """

    def has_permission(self, request, view):
        return bool(request.user and not request.user.is_authenticated)



from django.conf import settings
from drf_spectacular.utils import extend_schema

from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.exceptions import NotAuthenticated, ValidationError
from rest_framework.response import Response

from garpix_user.exceptions import NotAuthenticateException
from garpix_user.serializers import PhoneConfirmSendSerializer, PhoneConfirmCheckCodeSerializer, \
    PhonePreConfirmSendSerializer
from garpix_user.models import UserSession
from django.utils.translation import gettext_lazy as _

from garpix_user.utils.drf_spectacular import user_session_token_header_parameter


@extend_schema(
    parameters=[
        user_session_token_header_parameter()
    ]
)
class PhoneConfirmationView(viewsets.GenericViewSet):

    def get_serializer_class(self):
        user = self.request.user
        if self.action == 'send_code':
            if user.is_authenticated:
                return PhoneConfirmSendSerializer
            return PhonePreConfirmSendSerializer
        if self.action == 'check_code':
            return PhoneConfirmCheckCodeSerializer
        return PhoneConfirmCheckCodeSerializer

    @extend_schema(summary=_('Phone confirmation. Step 1'))
    @action(methods=['POST'], detail=False)
    def send_code(self, request, *args, **kwargs):
        user = request.user

        if user.is_authenticated:
            serializer = self.get_serializer(data=request.data)
            serializer.is_valid(raise_exception=True)

            result = user.send_phone_confirmation_code(serializer.data.get('phone', None))
            if result is not True:
                result.raise_exception(exception_class=ValidationError)
            return Response({'result': 'success'})
        else:
            if settings.GARPIX_USER.get('USE_PREREGISTRATION_PHONE_CONFIRMATION', False):
                user = UserSession.get_or_create_user_session(request)
                serializer = self.get_serializer(data=request.data)
                serializer.is_valid(raise_exception=True)

                result = user.send_phone_confirmation_code(serializer.data['phone'])
                if result is not True:
                    result.raise_exception(exception_class=ValidationError)
                return Response({'result': 'success'})

            raise NotAuthenticateException().raise_exception(exception_class=NotAuthenticated)

    @extend_schema(summary=_('Phone confirmation. Step 2'))
    @action(methods=['POST'], detail=False)
    def check_code(self, request, *args, **kwargs):
        user = request.user
        if not user.is_authenticated:
            if settings.GARPIX_USER.get('USE_PREREGISTRATION_PHONE_CONFIRMATION', False):
                user = UserSession.get_or_create_user_session(request)
            else:
                raise NotAuthenticateException().raise_exception(exception_class=NotAuthenticated)

        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        result = user.confirm_phone(serializer.data['phone_confirmation_code'])
        if result is not True:
            result.raise_exception(exception_class=ValidationError)
        return Response({'result': 'success'})



from django.conf import settings
from django.views.generic import RedirectView

from garpix_user.models import UserSession, ReferralType, ReferralUserLink


class ReferralLinkView(RedirectView):

    def get_redirect_url(self, *args, **kwargs):
        hash = self.kwargs.get('hash', None)
        user = UserSession.get_or_create_user_session(self.request)
        status = 'success'
        try:
            referral_instance = ReferralType.objects.get(referral_hash=hash)
            ReferralUserLink.objects.create(user=user, referral_type=referral_instance)
        except Exception:
            status = 'error'

        return f"{settings.GARPIX_USER.get('REFERRAL_REDIRECT_URL', '/')}?status={status}"



from django.contrib.auth import get_user_model
from rest_framework import serializers


class JWTDataSerializer(serializers.ModelSerializer):

    class Meta:
        model = get_user_model()
        fields = ('first_name', 'last_name', 'email',)



import json

from django.contrib.auth import authenticate, login, logout, get_user_model
from django.contrib.auth.mixins import UserPassesTestMixin
from django.http import JsonResponse, HttpResponseNotAllowed
from django.shortcuts import redirect
from django.views.generic.base import RedirectView
from django.views.generic import FormView
from django.http import HttpResponse
from garpix_utils.logs.enums.get_enums import Action, ActionResult
from garpix_utils.logs.loggers import ib_logger
from garpix_utils.logs.services.logger_iso import LoggerIso

from garpix_user.forms import LoginForm
from django.utils.translation import gettext_lazy as _


class LogoutView(RedirectView):

    def get_redirect_url(self):

        request = self.request
        logout(request)

        message = f'Пользователь {request.user.username} вышел из системы.'
        log = ib_logger.create_log(action=Action.user_logout.value,
                                   obj=get_user_model().__name__,
                                   obj_address=request.path,
                                   result=ActionResult.success,
                                   sbj=request.user.username,
                                   sbj_address=LoggerIso.get_client_ip(request),
                                   msg=message)

        ib_logger.write_string(log)

        return self.url


class LoginView(UserPassesTestMixin, FormView):
    http_method_names = ['post']

    def http_method_not_allowed(self, request, *args, **kwargs):
        if self.request.accepts('text/html'):
            return redirect(self.request.GET.get('next', '/') or '/')
        return HttpResponseNotAllowed(self._allowed_methods())

    def test_func(self):
        return not self.request.user.is_authenticated

    def handle_no_permission(self):
        if self.request.accepts('text/html'):
            return redirect(self.request.GET.get('next', '/') or '/')
        return HttpResponse({"__all__": [_("You are already authenticated")]}, content_type='application/json',
                            status=403)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['request'] = self.request
        if not kwargs.get('data'):
            kwargs['data'] = json.loads(getattr(self.request, 'body') or b'{}')
        return kwargs

    @staticmethod
    def get_form_class(**kwargs):
        return LoginForm

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context

    def _return_error(self, form, error_text):
        try:
            form.add_error(None, error_text)
        except TypeError:
            pass
        return self.form_invalid(form)

    def form_valid(self, form):

        request = self.request
        data = form.data
        username = data.get('username')
        password = data.get('password')
        user = authenticate(request, username=username.lower(), password=password)

        if user:
            user.set_user_session(request)
        login(request, user)

        if self.request.accepts('text/html'):
            return redirect(request.GET.get('next', '/') or '/')
        return JsonResponse({'success': True})

    def form_invalid(self, form):
        if self.request.accepts('text/html'):
            return self.render_to_response(self.get_context_data(form=form))
        return HttpResponse(json.dumps(form.errors), content_type='application/json', status=400)



from datetime import timedelta

from django.utils.translation import gettext_lazy as _

from rest_framework import serializers
from rest_framework.authentication import authenticate

from garpix_user.utils.current_date import set_current_date
from garpix_user.utils.get_password_settings import get_password_settings


class AuthTokenSerializer(serializers.Serializer):
    username = serializers.CharField(label=_("Username"))
    password = serializers.CharField(
        label=_("Password"),
        style={'input_type': 'password'},
        trim_whitespace=False
    )

    def validate(self, attrs):
        username = attrs.get('username')
        password = attrs.get('password')
        username = username.lower()

        if username and password:
            password_validity_period = get_password_settings()['password_validity_period']

            request = self.context.get('request')
            user = authenticate(request=request,
                                username=username, password=password)
            # The authenticate call simply returns None for is_active=False
            # users. (Assuming the default ModelBackend authentication
            # backend.)
            if not user or user.keycloak_auth_only:
                msg = _('Unable to log in with provided credentials.')
                raise serializers.ValidationError(msg, code='authorization')
            if user.is_blocked:
                msg = _(
                    'Your account is blocked. Please contact your administrator')
                raise serializers.ValidationError(msg, code='authorization')

            if password_validity_period != -1 and not user.keycloak_auth_only and user.password_updated_date + timedelta(
                    days=password_validity_period) <= set_current_date():
                msg = {
                    'non_field_errors': [
                        _('Your password has expired. Please change password')],
                    'extra_parameters': ['needs_password_update']
                }
                raise serializers.ValidationError(msg, code='authorization')

            if user.needs_password_update:
                msg = {
                    'non_field_errors': [
                        _('Your need to reset your password to activate your account')],
                    'extra_parameters': ['needs_password_update']
                }
                raise serializers.ValidationError(msg, code='authorization')

            user.set_user_session(request)
        else:
            msg = _('Must include "username" and "password".')
            raise serializers.ValidationError(msg, code='authorization')

        attrs['user'] = user
        return attrs



from django.contrib import admin
from garpix_utils.logs.mixins.log_admin import LogAdminMixin

from ..models import PasswordHistory


@admin.register(PasswordHistory)
class PasswordHistoryAdmin(LogAdminMixin):
    list_display = ['user', 'created_at']

    def has_delete_permission(self, request, obj=None):
        return True

    def has_change_permission(self, request, obj=None):
        return False



from datetime import datetime, timedelta

from celery.schedules import crontab
from django.conf import settings
from django.contrib.auth import get_user_model
from django.utils.module_loading import import_string
from garpix_notify.models import SystemNotify
from django.utils.translation import gettext_lazy as _
from garpix_user.utils.get_password_settings import get_password_settings
from garpix_user.utils.repluralize import rupluralize

celery_app = import_string(settings.GARPIXCMS_CELERY_SETTINGS)


@celery_app.task()
def password_validity_passed():
    password_settings = get_password_settings()
    _password_validity_period = password_settings['password_validity_period']
    _password_validity_inform_days = password_settings['password_validity_inform_days']
    if _password_validity_period != -1 and _password_validity_inform_days != -1:

        password_validity_period = datetime.now() - timedelta(
            days=(_password_validity_period - _password_validity_inform_days))
        inform_users = get_user_model().active_objects.filter(password_updated_date__lte=password_validity_period,
                                                              keycloak_auth_only=False)

        datenow = datetime.now()
        for user in inform_users:
            expire_days = (
                        user.password_updated_date + timedelta(days=_password_validity_period) - datenow).days

            if expire_days > 0:
                _msg = _(
                    'Your password will expire in {expire_days} {days}. Please change your password').format(
                    expire_days=1 + expire_days,
                    days=rupluralize(expire_days, _('day,days'))
                )
            else:
                _msg = _('Your password has expired. Please change your password')
            SystemNotify.send({
                'message': {
                    'message': str(_msg)
                },
                'event': settings.PASSWORD_INVALID_EVENT
            }, user, event=settings.PASSWORD_INVALID_EVENT, room_name=f'workflow-{user.pk}')


celery_app.conf.beat_schedule.update({
    'password_validity_passed': {
        'task': 'garpix_user.tasks.password_validity_passed.password_validity_passed',
        'schedule': crontab(minute='0', hour='21'),
    }
})
celery_app.conf.timezone = 'UTC'



from garpix_utils.logs.enums.get_enums import Action, ActionResult
from garpix_utils.logs.loggers import ib_logger
from garpix_utils.logs.mixins.create_log import CreateLogMixin
from garpix_utils.logs.services.logger_iso import LoggerIso
from garpix_utils.models import AdminDeleteMixin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.utils.translation import gettext_lazy as _

from garpix_user.utils.get_password_settings import get_password_settings


class UserAdmin(AdminDeleteMixin, BaseUserAdmin, CreateLogMixin):
    change_form_template = "garpix_user/send_confirm.html"

    fieldsets = (
        (None, {'fields': ('username', 'password')}),
        (_('Personal info'), {'fields': ('first_name', 'last_name', 'email', 'phone')}),
        (_('Permissions'), {
            'fields': (
                'is_active', 'is_deleted', 'is_staff', 'is_superuser',
                'is_blocked', 'login_attempts_count', 'password_updated_date', 'needs_password_update',
                'groups', 'user_permissions', 'keycloak_auth_only'
            ),
        }),
        (_('Viber'), {
            'fields': ('viber_chat_id', 'viber_secret_key',)
        }),
        (_('Telegram'), {
            'fields': ('telegram_chat_id', 'telegram_secret', 'get_telegram_connect_user_help'),
        }),
        (_('Confim information'), {
            'fields': (
                'is_email_confirmed', 'email_confirmation_code', 'is_phone_confirmed', 'phone_confirmation_code'),
        }),
    )
    readonly_fields = ['telegram_secret', 'get_telegram_connect_user_help'] + list(BaseUserAdmin.readonly_fields)

    def delete_model(self, request, obj):
        action = Action.user_delete.value
        log = self.log_delete(ib_logger, request, obj, action)
        super().delete_model(request, obj)
        ib_logger.write_string(log)

    def delete_queryset(self, request, queryset):
        action = Action.user_delete.value
        logs = []
        for obj in queryset:
            logs.append(self.log_delete(ib_logger, request, obj, action))
        super().delete_queryset(request, queryset)
        for log in logs:
            ib_logger.write_string(log)

    def save_model(self, request, obj, form, change):
        password_first_change = get_password_settings()['password_first_change']

        if not change and password_first_change:
            obj.needs_password_update = True

        log = self.log_change_or_create(ib_logger, request, obj, change,
                                        action_change=Action.user_change.value,
                                        action_create=Action.user_create.value)
        ib_logger.write_string(log)
        super().save_model(request, obj, form, change)

    def save_related(self, request, form, formsets, change):
        obj = form.instance
        prev_groups, prev_permissions = set(), set()
        if change:
            prev_groups = set([str(_obj) for _obj in obj.groups.all()])
            prev_permissions = set([str(_obj) for _obj in obj.user_permissions.all()])
            log = self.log_change_m2m_field(ib_logger, request, super(), form, formsets, change,
                                            action_change=Action.user_change.value,
                                            exclude_fields=['groups', 'user_permissions'])
            if log:
                ib_logger.write_string(log)

        else:
            super().save_related(request, form, formsets, change)

        groups = set([str(_obj) for _obj in obj.groups.all()])
        permissions = set([str(_obj) for _obj in obj.user_permissions.all()])

        new_old_groups = groups - prev_groups
        old_new_groups = prev_groups - groups

        new_old_permissions = permissions - prev_permissions
        old_new_permissions = prev_permissions - permissions

        if new_old_groups or old_new_groups:
            if new_old_groups:
                msg = f'Пользователь {str(obj)}(id={obj.pk}) был добавлен в группы {new_old_groups}'
                log = ib_logger.create_log(action=Action.group_add_user.value,
                                           obj=obj.__class__.__name__,
                                           obj_address=request.path,
                                           result=ActionResult.success,
                                           sbj=request.user.username,
                                           sbj_address=LoggerIso.get_client_ip(request),
                                           msg=msg)
                ib_logger.write_string(log)
            if old_new_groups:
                msg = f'Пользователь {str(obj)}(id={obj.pk}) был удален из групп {old_new_groups}'
                log = ib_logger.create_log(action=Action.group_delete_user.value,
                                           obj=obj.__class__.__name__,
                                           obj_address=request.path,
                                           result=ActionResult.success,
                                           sbj=request.user.username,
                                           sbj_address=LoggerIso.get_client_ip(request),
                                           msg=msg)
                ib_logger.write_string(log)

        msg = CreateLogMixin.log_msg_change if CreateLogMixin.log_msg_change else f'Привилегии пользователя {str(obj)}(id={obj.pk}) были изменены'

        changed_fields = ''
        if new_old_permissions or old_new_permissions:
            if new_old_permissions:
                changed_fields += f'+добавлены {new_old_permissions} '
            if old_new_permissions:
                changed_fields += f'-удалены {old_new_permissions} '

            log = ib_logger.create_log(action=Action.user_access.value,
                                       obj=obj.__class__.__name__,
                                       obj_address=request.path,
                                       result=ActionResult.success,
                                       sbj=request.user.username,
                                       params=changed_fields,
                                       sbj_address=LoggerIso.get_client_ip(request),
                                       msg=msg)
            ib_logger.write_string(log)



from django.core.validators import BaseValidator
from django.utils.translation import gettext_lazy as _


class PositiveWithInfValidator(BaseValidator):
    message = _('Ensure this value is a positive integer or -1')

    def compare(self, a, b):
        return a <= 0 and a != -1



from rest_framework import parsers, renderers, status

from garpix_user.models.access_token import AccessToken as Token
from garpix_user.serializers.refresh_token_serializer import RefreshTokenSerializer
from rest_framework.response import Response
from rest_framework.views import APIView
from garpix_user.models.refresh_token import RefreshToken
from django.utils import timezone
from datetime import timedelta

from garpix_user.utils.get_password_settings import get_password_settings


class RefreshTokenView(APIView):
    throttle_classes = ()
    permission_classes = ()
    parser_classes = (parsers.FormParser, parsers.MultiPartParser, parsers.JSONParser,)
    renderer_classes = (renderers.JSONRenderer,)
    serializer_class = RefreshTokenSerializer

    def post(self, request, *args, **kwargs):
        refresh_token_ttl_seconds = get_password_settings()['refresh_token_ttl_seconds']
        access_token_ttl_seconds = get_password_settings()['access_token_ttl_seconds']
        serializer = self.serializer_class(data=request.data, context={'request': request})
        serializer.is_valid(raise_exception=True)
        refresh_token = serializer.validated_data['refresh_token']
        try:
            refresh_token_obj = RefreshToken.objects.get(key=refresh_token)
            if refresh_token_ttl_seconds > 0:
                if refresh_token_obj.created + timedelta(seconds=refresh_token_ttl_seconds) < timezone.now():
                    refresh_token_obj.delete()
                    raise Exception("Token expired.")
            token = Token.objects.create(user=refresh_token_obj.user)
            return Response({
                'access_token': token.key,
                'access_token_expires': access_token_ttl_seconds,
                'result': True,
            })
        except:  # noqa
            return Response({'result': False}, status=status.HTTP_400_BAD_REQUEST)


refresh_token_view = RefreshTokenView.as_view()



from django.conf import settings


class CodeLengthMixin:

    def get_confirm_code_length(self, type='email'):
        GARPIX_USER_SETTINGS = getattr(settings, "GARPIX_USER", {})

        if type == 'email':

            MAX_LENGTH, CONFIRM_CODE_LENGTH = 255, GARPIX_USER_SETTINGS.get('CONFIRM_EMAIL_CODE_LENGTH', 6)
        else:
            MAX_LENGTH, CONFIRM_CODE_LENGTH = 15, GARPIX_USER_SETTINGS.get('CONFIRM_PHONE_CODE_LENGTH', 6)

        code_length = 4 if CONFIRM_CODE_LENGTH < 4 else MAX_LENGTH if CONFIRM_CODE_LENGTH > MAX_LENGTH else CONFIRM_CODE_LENGTH

        return code_length

    class Meta:
        abstract = True



from django.db import models
from django.utils.translation import gettext_lazy as _
from garpix_utils.string import get_random_string

from .user_session import UserSession


class ReferralType(models.Model):
    title = models.CharField(max_length=128, verbose_name=_('Referral way title'))
    referral_hash = models.CharField(max_length=32)

    def save(self, force_insert=False, force_update=False, using=None,
             update_fields=None):
        if self.pk is None:
            self.referral_hash = get_random_string(32)
        super().save(force_insert, force_update, using, update_fields)


class ReferralUserLink(models.Model):
    user = models.ForeignKey(UserSession, on_delete=models.CASCADE, verbose_name=_('User'))
    referral_type = models.ForeignKey(ReferralType, on_delete=models.CASCADE,
                                      verbose_name=_('Where did the user come from'))

    class Meta:
        unique_together = (('user', 'referral_type'),)



from django.conf import settings


def get_token_from_request(request, keyword='Bearer'):
    header_key = settings.GARPIX_USER.get('REST_AUTH_HEADER_KEY', 'HTTP_AUTHORIZATION')
    if header_key not in request.META:
        return None

    token = request.META[header_key]
    token = token[len(keyword) + 1:]
    return token



from django.contrib import admin
from garpix_utils.logs.mixins.log_admin_solo import LogAdminSolo
from solo.admin import SingletonModelAdmin
from garpix_user.models import GarpixUserPasswordConfiguration


@admin.register(GarpixUserPasswordConfiguration)
class GarpixUserPasswordConfigurationAdmin(LogAdminSolo, SingletonModelAdmin):
    pass



from .registration_serializer import RegistrationSerializer  # noqa
from .email_confirmation_serializer import (  # noqa
    EmailConfirmSendSerializer,
    EmailConfirmCheckCodeSerializer,
    EmailPreConfirmSendSerializer
)
from .auth_token_serializer import AuthTokenSerializer  # noqa
from .refresh_token_serializer import RefreshTokenSerializer  # noqa
from .phone_confirmation_serializer import (  # noqa
    PhoneConfirmSendSerializer,
    PhoneConfirmCheckCodeSerializer,
    PhonePreConfirmSendSerializer
)
from .passwrod_serializer import RestorePasswordSerializer, RestoreSetPasswordSerializer, RestoreCheckCodeSerializer, \
    ChangePasswordSerializer  # noqa

from .user_session_serializer import UserSessionSerializer, UserSessionTokenSerializer  # noqa
from .jwt_data_serializer import JWTDataSerializer  # noqa



from django.conf import settings
from django.db import IntegrityError
from django.utils.module_loading import import_string
from rest_framework.fields import DateTimeField
from rest_framework.response import Response
from garpix_user.models.access_token import AccessToken as Token
from garpix_user.models.refresh_token import RefreshToken
from garpix_user.utils.current_date import set_current_date
from django.utils.translation import gettext as _
import jwt

from garpix_user.utils.get_password_settings import get_password_settings


class AuthTokenViewMixin:

    def _get_access_token_data(self, user):
        _password_settings = get_password_settings()

        if (access_tokens_count := _password_settings['access_tokens_count']) != -1:
            _prev_tokens = Token.objects.filter(user=user).order_by('-created')[
                           :access_tokens_count - 1].values_list('pk', flat=True)
            Token.objects.exclude(pk__in=_prev_tokens).delete()

        token = Token.objects.create(user=user)
        refresh_token = RefreshToken.objects.create(user=user)
        return Response({
            'access_token': token.key,
            'refresh_token': refresh_token.key,
            'token_type': 'Bearer',
            'access_token_expires': _password_settings['access_token_ttl_seconds'],
            'refresh_token_expires': _password_settings['refresh_token_ttl_seconds'],
        })

    def _get_jwt_data(self, user):
        access_token_ttl_seconds = get_password_settings()['access_token_ttl_seconds']
        jwt_secret_key = settings.GARPIX_USER.get('JWT_SECRET_KEY', None)

        if jwt_secret_key is None:
            raise IntegrityError(_('JWT_SECRET_KEY is not set'))

        serializer = import_string(
            settings.GARPIX_USER.get('JWT_SERIALIZER', 'garpix_user.serializers.JWTDataSerializer'))

        token = jwt.encode(
            {
                'created_at': DateTimeField().to_representation(set_current_date()),
                'username': user.username,
                'user_data': serializer(user).data
            },
            jwt_secret_key,
            algorithm='HS256'
        )

        return Response({
            'access_token': token,
            'token_type': 'Bearer',
            'access_token_expires': access_token_ttl_seconds
        })



AUTHENTICATION_BACKENDS = (
    'social_core.backends.google.GoogleOAuth2',
    'social_core.backends.twitter.TwitterOAuth',
    'social_core.backends.vk.VKOAuth2',
    'social_core.backends.facebook.FacebookAppOAuth2',
    'social_core.backends.facebook.FacebookOAuth2',
    # Django
    'rest_framework_social_oauth2.backends.DjangoOAuth2',
    'django.contrib.auth.backends.ModelBackend',
)
SOCIAL_AUTH_PIPELINE = (
    'social_core.pipeline.social_auth.social_details',
    'social_core.pipeline.social_auth.social_uid',
    'social_core.pipeline.social_auth.auth_allowed',
    'social_core.pipeline.social_auth.social_user',
    'social_core.pipeline.user.get_username',
    'social_core.pipeline.social_auth.associate_by_email',
    'social_core.pipeline.user.create_user',
    'social_core.pipeline.social_auth.associate_user',
    'social_core.pipeline.social_auth.load_extra_data',
    'social_core.pipeline.user.user_details'
)

PHONE_CONFIRMATION_EVENT = 4210
EMAIL_CONFIRMATION_EVENT = 4211

RESTORE_PASSWORD_EMAIL_EVENT = 4212
RESTORE_PASSWORD_PHONE_EVENT = 4213
EMAIL_LINK_CONFIRMATION_EVENT = 4214
PASSWORD_INVALID_EVENT = 4215

GARPIX_USER_NOTIFY_EVENTS = {
    PHONE_CONFIRMATION_EVENT: {
        'title': 'Подтверждение номера телефона',
        'context_description': '{{ confirmation_code }}'
    },

    EMAIL_CONFIRMATION_EVENT: {
        'title': 'Подтверждение email',
        'context_description': '{{ confirmation_code }}'
    },

    RESTORE_PASSWORD_EMAIL_EVENT: {
        'title': 'Восстановление пароля по email',
        'context_description': '{{ restore_code }} - restore code, {{ user }} - user instance'
    },

    RESTORE_PASSWORD_PHONE_EVENT: {
        'title': 'Восстановление пароля по номеру телефона',
        'context_description': '{{ restore_code }} - restore code, {{ user }} - user instance'
    },

    EMAIL_LINK_CONFIRMATION_EVENT: {
        'title': 'Подтверждение email по ссылке',
        'context_description': '{{ confirmation_link }}'
    },
    PASSWORD_INVALID_EVENT: {
        'title': 'Уведомление об истечении срока действия пароля',
        'context_description': '{{ message }} - Сообщение, ',
        'event_description': 'Уведомление об истечении срока действия пароля',
    },
}



from django.contrib.auth import get_user_model
from garpix_utils.logs.enums.get_enums import Action, ActionResult
from garpix_utils.logs.loggers import ib_logger
from garpix_utils.logs.services.logger_iso import LoggerIso
from rest_framework import parsers, renderers
from garpix_user.models.access_token import AccessToken as Token
from oauth2_provider.models import AccessToken
from rest_framework.response import Response
from rest_framework.views import APIView
from garpix_user.models.refresh_token import RefreshToken
from rest_framework.permissions import IsAuthenticated
from garpix_user.utils import get_token_from_request


class LogoutView(APIView):
    throttle_classes = ()
    permission_classes = (IsAuthenticated,)
    parser_classes = (parsers.FormParser, parsers.MultiPartParser, parsers.JSONParser,)
    renderer_classes = (renderers.JSONRenderer,)

    def post(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            token = get_token_from_request(request)
            if token is not None:
                Token.objects.filter(key=token).delete()
                AccessToken.objects.filter(token=token).delete()
                RefreshToken.objects.filter(key=token).delete()

                message = f'Пользователь {request.user.username} вышел из системы.'
                log = ib_logger.create_log(action=Action.user_logout.value,
                                           obj=get_user_model().__name__,
                                           obj_address=request.path,
                                           result=ActionResult.success,
                                           sbj=request.user.username,
                                           sbj_address=LoggerIso.get_client_ip(request),
                                           msg=message)

                ib_logger.write_string(log)

                return Response({
                    'result': True,
                })

        return Response({
            'result': False,
        }, status=401)


logout_view = LogoutView.as_view()
